@@ -5,28 +5,48 @@
 // SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 
 #include "notification-manager.h"
+#include "config.h"
 
+#include <KIO/ApplicationLauncherJob>
 #include <KLocalizedString>
-#include <MprisQt/MprisController>
+#include <QDBusConnection>
 #include <QTimer>
 
-static void waitForControllerInit(MprisController *mprisController)
+static bool getScreenSaverActive()
 {
-    qDebug() << Q_FUNC_INFO << mprisController->service() << mprisController->isValid();
-    QTimer timer; // as a precaution
-    timer.setSingleShot(true);
-    QEventLoop loop;
-    QObject::connect(mprisController, &MprisController::playbackStatusChanged, &loop, &QEventLoop::quit);
-    QObject::connect(&timer, &QTimer::timeout, &loop, &QEventLoop::quit);
-    timer.start(300);
-    loop.exec();
+    bool active = false;
+#ifdef DIALER_BUILD_SHELL_OVERLAY
+    QDBusMessage request = QDBusMessage::createMethodCall(QStringLiteral("org.freedesktop.ScreenSaver"),
+                                                          QStringLiteral("/ScreenSaver"),
+                                                          QStringLiteral("org.freedesktop.ScreenSaver"),
+                                                          QStringLiteral("GetActive"));
+    const QDBusReply<bool> response = QDBusConnection::sessionBus().call(request);
+    active = response.isValid() ? response.value() : false;
+#endif // DIALER_BUILD_SHELL_OVERLAY
+    return active;
+}
+
+static void launchPlasmaDialerDesktopFile()
+{
+    const auto desktopName = QStringLiteral("org.kde.phone.dialer");
+    const KService::Ptr appService = KService::serviceByDesktopName(desktopName);
+    if (!appService) {
+        qWarning() << "Could not find" << desktopName;
+        return;
+    }
+    KIO::ApplicationLauncherJob job(appService);
+    job.start();
 }
 
 NotificationManager::NotificationManager(QObject *parent)
     : QObject(parent)
+    , _ringingNotification(std::make_unique<KNotification>(QStringLiteral("ringing"), KNotification::Persistent | KNotification::LoopSound, nullptr))
+
+#ifdef HAVE_QT5_FEEDBACK
     , _ringEffect(std::make_unique<QFeedbackHapticsEffect>())
-    , _mprisManager(this)
+#endif // HAVE_QT5_FEEDBACK
 {
+#ifdef HAVE_QT5_FEEDBACK
     _ringEffect->setAttackIntensity(0.1);
     _ringEffect->setAttackTime(420);
     _ringEffect->setIntensity(0.7);
@@ -34,6 +54,18 @@ NotificationManager::NotificationManager(QObject *parent)
     _ringEffect->setFadeTime(700);
     _ringEffect->setFadeIntensity(0.07);
     _ringEffect->setPeriod(1300);
+#endif // HAVE_QT5_FEEDBACK
+
+    _databaseInterface = new org::kde::telephony::CallHistoryDatabase(QString::fromLatin1(_databaseInterface->staticInterfaceName()),
+                                                                      QStringLiteral("/org/kde/telephony/CallHistoryDatabase/tel/mm"),
+                                                                      QDBusConnection::sessionBus(),
+                                                                      this);
+
+    if (!_databaseInterface->isValid()) {
+        qDebug() << Q_FUNC_INFO << "Could not initiate CallHistoryDatabase interface";
+        return;
+    }
+    _ringingNotification->setAutoDelete(false);
 }
 
 void NotificationManager::setCallUtils(org::kde::telephony::CallUtils *callUtils)
@@ -69,7 +101,7 @@ void NotificationManager::onCallAdded(const QString &deviceUni,
 void NotificationManager::onCallDeleted(const QString &deviceUni, const QString &callUni)
 {
     qDebug() << Q_FUNC_INFO << "call deleted" << deviceUni << callUni;
-    closeRingingNotification();
+    handleCallInteraction();
 }
 
 void NotificationManager::onCallStateChanged(const QString &deviceUni,
@@ -81,50 +113,43 @@ void NotificationManager::onCallStateChanged(const QString &deviceUni,
     qDebug() << Q_FUNC_INFO << "call state changed:" << deviceUni << callUni << callDirection << callState << callStateReason;
     if (callDirection == DialerTypes::CallDirection::Incoming) {
         if (callState == DialerTypes::CallState::Terminated) {
-            handleRejectedCall();
+            handleCallInteraction();
+        }
+        if (callState == DialerTypes::CallState::Active) {
+            handleCallInteraction();
         }
     }
 }
 
-void NotificationManager::openRingingNotification(const QString &deviceUni, const QString &callUni, const QString communicationWith)
+void NotificationManager::openRingingNotification(const QString &deviceUni,
+                                                  const QString &callUni,
+                                                  const QString callerDisplay,
+                                                  const QString notificationEvent)
 {
-    _ringEffect->start();
-
-    QString contactName = _contactUtils->displayString(communicationWith);
-    QString callerDisplayString =
-        (contactName == communicationWith) ? communicationWith : communicationWith + QStringLiteral("<br>") + QStringLiteral("<b>%1</b>").arg(contactName);
-
     QStringList actions;
     actions << i18n("Accept") << i18n("Reject");
-    qDebug() << Q_FUNC_INFO << _ringingNotification;
-    if (!_ringingNotification) {
-        qDebug() << Q_FUNC_INFO << "new notification";
-        _ringingNotification = new KNotification(QStringLiteral("ringing"), KNotification::Persistent | KNotification::LoopSound, nullptr);
-    }
+
+    _ringingNotification->setEventId(notificationEvent);
     _ringingNotification->setUrgency(KNotification::CriticalUrgency);
     _ringingNotification->setComponentName(QStringLiteral("plasma_dialer"));
+
     // _ringingNotification->setPixmap(person.photo());
     _ringingNotification->setTitle(i18n("Incoming call"));
-    _ringingNotification->setText(callerDisplayString);
+    _ringingNotification->setText(callerDisplay);
     // this will be used by the notification applet to show custom notification UI
     // with swipe decision.
     _ringingNotification->setHint(QStringLiteral("category"), QStringLiteral("x-kde.incoming-call"));
     _ringingNotification->setActions(actions);
     _ringingNotification->addContext(QStringLiteral("deviceUni"), deviceUni);
     _ringingNotification->addContext(QStringLiteral("callUni"), callUni);
-    connect(_ringingNotification, QOverload<unsigned int>::of(&KNotification::activated), this, &NotificationManager::onNotificationAction);
+    connect(_ringingNotification.get(), QOverload<unsigned int>::of(&KNotification::activated), this, &NotificationManager::onNotificationAction);
     _ringingNotification->sendEvent();
 }
 
 void NotificationManager::closeRingingNotification()
 {
-    if (!_ringingNotification) {
-        return;
-    }
     _ringingNotification->disconnect();
     _ringingNotification->close();
-    _ringingNotification = nullptr;
-    _ringEffect->stop();
 }
 
 void NotificationManager::accept(const QString &deviceUni, const QString &callUni)
@@ -147,54 +172,94 @@ void NotificationManager::hangUp(const QString &deviceUni, const QString &callUn
     closeRingingNotification();
 }
 
-void NotificationManager::handleIncomingCall(const QString &deviceUni, const QString &callUni, const QString communicationWith)
+void NotificationManager::handleIncomingCall(const QString &deviceUni, const QString &callUni, const QString &communicationWith)
 {
-    pauseMedia();
-    openRingingNotification(deviceUni, callUni, communicationWith);
-}
+    const QString contactName = _contactUtils->displayString(communicationWith);
 
-void NotificationManager::handleRejectedCall()
-{
-    closeRingingNotification();
-    unpauseMedia();
-}
+    bool allowed = true;
 
-void NotificationManager::pauseMedia()
-{
-    if (_mprisManager.playbackStatus() != Mpris::Playing) {
-        return;
-    }
+    if (Config::self()->adaptiveBlocking() && contactName == communicationWith) {
+        allowed = false;
 
-    auto services = _mprisManager.availableServices();
-    for (auto service : services) {
-        qDebug() << service;
-        if (service.contains(QLatin1String("kdeconnect"))) {
-            qDebug() << Q_FUNC_INFO << "Skip players connected over KDE Connect. KDE Connect pauses them itself";
-            continue;
+        if (Config::self()->allowAnonymous() && communicationWith.isEmpty()) {
+            allowed = true;
         }
-        MprisController mprisController(service, QDBusConnection::sessionBus(), this);
-        waitForControllerInit(&mprisController);
-        if (mprisController.playbackStatus() == Mpris::Playing) {
-            if (!_pausedSources.contains(service)) {
-                _pausedSources.insert(service);
-                if (mprisController.canPause()) {
-                    mprisController.pause();
-                } else {
-                    mprisController.stop();
+
+        if (Config::self()->allowPreviousOutgoing()) {
+            QString lastOutgoing = _databaseInterface->lastCall(communicationWith, static_cast<int>(DialerTypes::CallDirection::Outgoing));
+            if (!lastOutgoing.isEmpty()) {
+                allowed = true;
+            }
+        }
+
+        if (Config::self()->allowCallback()) {
+            QString lastIncoming = _databaseInterface->lastCall(communicationWith, static_cast<int>(DialerTypes::CallDirection::Incoming));
+            QDateTime lastTime = QDateTime::fromString(lastIncoming, QStringLiteral("yyyy-MM-ddThh:mm:ss.zzz"));
+            qint64 diff = lastTime.msecsTo(QDateTime::currentDateTime());
+            if (diff / 1000 / 60 < Config::self()->callbackInterval()) {
+                allowed = true;
+            }
+        }
+
+        if (Config::self()->allowedPatterns().length() > 0) {
+            for (const auto &pattern : Config::self()->allowedPatterns()) {
+                if (communicationWith.indexOf(pattern) >= 0) {
+                    allowed = true;
+                    break;
                 }
             }
         }
     }
+
+    QString notificationEvent = QStringLiteral("ringing");
+
+    if (!allowed) {
+        if (Config::self()->ringOption() == 0) {
+            hangUp(deviceUni, callUni);
+            return;
+        } else if (Config::self()->ringOption() == 1) {
+            return;
+        }
+        notificationEvent = QStringLiteral("ringing-silent");
+    }
+
+    if (allowed) {
+        startHapticsFeedback();
+    }
+
+    QString callerDisplay =
+        (contactName == communicationWith) ? communicationWith : communicationWith + QStringLiteral("<br>") + QStringLiteral("<b>%1</b>").arg(contactName);
+
+    if (callerDisplay.isEmpty()) {
+        callerDisplay = i18n("No Caller ID");
+    }
+
+    bool screenLocked = getScreenSaverActive();
+    if (screenLocked) {
+        launchPlasmaDialerDesktopFile();
+    } else {
+        openRingingNotification(deviceUni, callUni, callerDisplay, notificationEvent);
+    }
 }
 
-void NotificationManager::unpauseMedia()
+void NotificationManager::handleCallInteraction()
 {
-    for (const QString &service : qAsConst(_pausedSources)) {
-        MprisController mprisController(service, QDBusConnection::sessionBus(), this);
-        waitForControllerInit(&mprisController);
-        mprisController.play();
-    }
-    _pausedSources.clear();
+    stopHapticsFeedback();
+    closeRingingNotification();
+}
+
+void NotificationManager::startHapticsFeedback()
+{
+#ifdef HAVE_QT5_FEEDBACK
+    _ringEffect->start();
+#endif // HAVE_QT5_FEEDBACK
+}
+
+void NotificationManager::stopHapticsFeedback()
+{
+#ifdef HAVE_QT5_FEEDBACK
+    _ringEffect->stop();
+#endif // HAVE_QT5_FEEDBACK
 }
 
 void NotificationManager::onNotificationAction(unsigned int action)
@@ -214,7 +279,7 @@ void NotificationManager::onNotificationAction(unsigned int action)
     switch (action) {
     case 1:
         accept(deviceUni, callUni);
-        QProcess::startDetached(QStringLiteral("plasmaphonedialer"), QStringList{});
+        launchPlasmaDialerDesktopFile();
         break;
     case 2:
         hangUp(deviceUni, callUni);
